<?php

/**
 * SecureTrading STPP Shopping Carts
 * Abantecart 0.9.1
 * Module Version 2.5.7
 * Last Updated 13 March 2013
 * Written by Peter Barrow for SecureTrading Ltd.
 * http://www.securetrading.com
 */

?><?php
 
if ( !defined ( 'DIR_CORE' )) {
	header ( 'Location: static_pages/' );
}

require_once(DIR_EXT . 'default_securetrading_ppages/securetrading_stpp/STPPLoader.php');
require_once(DIR_EXT . 'default_securetrading_ppages/AbantePPages.class.php');

class ControllerResponsesExtensionDefaultSecuretradingPpages extends AController {

	/**
	 * Prepares the credit card form for display:
	 */
	public function main() {
		$this->loadLanguage('default_securetrading_stapi/default_securetrading_ppages');
		
		$this->load->model('checkout/order');
		
		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
		
		$requestObject = new stdClass;
		$requestObject->sitereference = $this->config->get('default_securetrading_ppages_site_reference');
		$requestObject->currencyiso3a = $order_info['currency'];
		$requestObject->mainamount = AbantePPages::formatPrice($order_info['total'], $order_info['currency']);
		
		$requestObject->billingfirstname = $order_info['payment_firstname'];
		$requestObject->billinglastname = $order_info['payment_lastname'];
		$requestObject->billingtelephonetype = '';
		$requestObject->billingtelephone = $order_info['telephone'];
		$requestObject->billingemail = $order_info['email'];
		
		$requestObject->billingpremise = $order_info['payment_address_1'];
		$requestObject->billingstreet = $order_info['payment_address_2'];
		$requestObject->billingtown = $order_info['payment_city'];
		$requestObject->billingcounty = $order_info['payment_zone_code'];
		$requestObject->billingpostcode = $order_info['payment_postcode'];
		$requestObject->billingcountry = $order_info['payment_iso_code_2'];
		
		$requestObject->customerpremise = $order_info['shipping_firstname'];
		$requestObject->customerstreet = $order_info['shipping_lastname'];
		$requestObject->customertown = $order_info['shipping_city'];
		$requestObject->customercounty = $order_info['shipping_zone_code'];
		$requestObject->customerpostcode = $order_info['shipping_postcode'];
		$requestObject->customercountry = $order_info['shipping_iso_code_2'];
		
		$requestObject->settleduedate = $this->config->get('default_securetrading_ppages_settle_due_date');
		$requestObject->settlestatus = $this->config->get('default_securetrading_ppages_settle_status');
		
		$requestObject->orderreference = $order_info['order_id'];
		
		if ($this->config->get('default_securetrading_ppages_use_site_security')) {
			AbantePPages::createHash($requestObject, $this->config->get('default_securetrading_ppages_site_security'));
		}
		
		$ppages = new AbantePPages();
		$ppages->preChecks($requestObject);
		unset($ppages);
		
		$hiddenInputString = '';
		
		foreach($requestObject as $k => $v) {
			$hiddenInputString .= "<input type='hidden' name='{$k}' value='{$v}' />";
		}
		
		$template_data['hidden_fields'] = $hiddenInputString;
		
		$template_data['action_url'] = AbantePPages::getUrl();
		
		if ($this->request->get['rt'] != 'checkout/guest_step_3') {
			$template_data['back'] = $this->html->getSecureURL('checkout/payment');
		} else {
			$template_data['back'] = $this->html->getSecureURL('checkout/guest_step_2');
		}
		
		$this->view->batchAssign( $template_data );
		$this->processTemplate('responses/default_securetrading_ppages.tpl' );
	}
	
	/**
	 * Notification script.  Accessed via <root_abante_dir>/index.php?rt=extension/default_securetrading_ppages/notification
	 */
	public function notification() {
		
		// Instantiate new AbantePPages object:
		$ppages = new AbantePPages();
		
		// Ensure all required information has been returned to the script:
		
		if (!isset($_POST['errorcode'])) {
			$ppages->createException(new Exception('The errorcode has not been returned to the notification script.'), __FILE__, __CLASS__, __LINE__);
		}
		
		if (!isset($_POST['orderreference'])) {
			$ppages->createException(new Exception('The orderreference has not been returned to the notification script.'), __FILE__, __CLASS__, __LINE__);
		}
		
		if (!isset($_POST['securityresponseaddress']) || !isset($_POST['securityresponsepostcode']) || !isset($_POST['securityresponsesecuritycode'])) {
			$ppages->createException(new Exception('The security response information has not been correctly returned to the notification script.'), __FILE__, __CLASS__, __LINE__);
		}
		
		// Validate the transaction:
		
		if ($this->config->get('default_securetrading_ppages_use_notification_hash') && !isset($_POST['responsesitesecurity'])) {
			$ppages->createException(new Exception('The notification hash is enabled in the configuration but has not been returned to the notification script.'), __FILE__, __CLASS__, __LINE__);
		}
		
		if ($this->config->get('default_securetrading_ppages_use_notification_hash') && hash('sha256', $_POST['errorcode'] . $_POST['orderreference'] . $_POST['securityresponseaddress'] . $_POST['securityresponsepostcode'] . $_POST['securityresponsesecuritycode'] . $this->config->get('default_securetrading_ppages_notification_hash')) !== $_POST['responsesitesecurity']) {
			$ppages->createException(new Exception('The notification hash has been returned to the notification script but it did not match the regenerated hash.'), __FILE__, __CLASS__, __LINE__);
		}
		
		// Prepare AVS information:
		$securityAddress = $params['result']->securityaddress ? $params['result']->securityaddress : 'Not Matched';
		$securityPostcode = $params['result']->securitypostcode ? $params['result']->securitypostcode : 'Not Matched';
		$securityCode = $params['result']->securitycode ? $params['result']->securitycode : 'Not Matched';
		
		// Create AVS String:
		$avsString = "
			AVS Address: {$securityAddress}\n
			AVS Postcode: {$securityPostcode}\n
			AVS Security Code: {$securityCode}
		";
		
		// Instantiate models:
		$this->load->model('checkout/order');
		$this->load->model('extension/default_securetrading_ppages');
		
		// Update the order in different ways depending on the errorcode returned:
		if ($_POST['errorcode'] == 0) {
			
			// Retrieve the successful order status:
			$orderStatus = $this->config->get('default_securetrading_ppages_successful_order_status_id');
			
			// Reset the order status to 0.  This will allow the below _confirm() method to work if the order previously failed/declined:
			$this->model_extension_default_securetrading_ppages->resetOrderStatus($_POST['orderreference']);
			
			// Update the order status, update stock quantities, notify customer, etc:
			$this->model_checkout_order->_confirm($_POST['orderreference'], $orderStatus, $avsString);
		}
		else {
			
			// Retrieve the order status:
			$orderStatus = $_POST['errorcode'] == 70000 ? (int) $this->config->get('default_securetrading_ppages_declined_order_status_id') : $this->config->get('default_securetrading_ppages_failed_order_status_id');
			
			// Update the order status:
			$this->model_extension_default_securetrading_ppages->updateFailedOrder($_POST['orderreference'], $orderStatus);
		}
	}
	
	/**
	 * Redirect script.  Accessed via <root_abante_dir>/index.php?rt=extension/default_securetrading_ppages/redirect
	 */
	public function redirect() {
	
		// Instantiate new AbantePPages object:
		$ppages = new AbantePPages();
		
		// Validate the identity of the individual accessing this method:
		if ($this->config->get('default_securetrading_ppages_use_site_security') && !isset($_GET['responsesitesecurity'])) {
			$ppages->createException(new Exception('The site security hash is enabled but the responsesitesecurity field has not been returned to the redirect script.'), __FILE__, __CLASS__, __LINE__);
		}
		
		if ($this->config->get('default_securetrading_ppages_use_site_security') && hash('sha256', $this->config->get('default_securetrading_ppages_site_security')) !== $_GET['responsesitesecurity']) {
			$ppages->createException(new Exception('The responsesitesecurity field returned with the redirect did not match the regenerated hash.'), __FILE__, __CLASS__, __LINE__);
		}
		
		// Redirect the user and terminate the script:
		header("Location: " . HTTP_SERVER . INDEX_FILE . '?rt=checkout/success');
		exit;
	}
}
?>